@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2011-2023 The Octave Project Developers
+// Copyright (C) 2011-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -26,23 +26,10 @@
 #if ! defined (octave_GLCanvas_h)
 #define octave_GLCanvas_h 1
 
-#if defined (HAVE_QOPENGLWIDGET)
-#  include <QOpenGLWidget>
-#  define OCTAVE_QT_OPENGL_WIDGET QOpenGLWidget
-#  include <QOpenGLFramebufferObject>
-#  define OCTAVE_QT_OPENGL_FBO QOpenGLFramebufferObject
-#  if defined (HAVE_QT_OFFSCREEN)
-#    include <QOpenGLContext>
-#    include <QOffscreenSurface>
-#  endif
-#elif defined (HAVE_QGLWIDGET)
-#  include <QGLWidget>
-#  define OCTAVE_QT_OPENGL_WIDGET QGLWidget
-#  include <QGLFramebufferObject>
-#  define OCTAVE_QT_OPENGL_FBO QGLFramebufferObject
-#else
-#  error "configuration error: must have <QOpenGLWidget> or <QGLWidget>."
-#endif
+#include <QOffscreenSurface>
+#include <QOpenGLContext>
+#include <QOpenGLFramebufferObject>
+#include <QOpenGLWidget>
 
 #include "Canvas.h"
 
@@ -51,33 +38,34 @@
 
 OCTAVE_BEGIN_NAMESPACE(octave)
 
-class base_qobject;
-}
-
-OCTAVE_BEGIN_NAMESPACE(octave)
-
-class GLCanvas : public OCTAVE_QT_OPENGL_WIDGET, public Canvas
+class GLWidget : public QOpenGLWidget
 {
+  Q_OBJECT
+
 public:
-  GLCanvas (octave::base_qobject& oct_qobj, octave::interpreter& interp,
-            const graphics_handle& handle, QWidget *parent);
-  ~GLCanvas (void);
 
-  void initializeGL (void);
+  GLWidget (Canvas& parent_canvas, QWidget *parent);
 
-  void draw (const graphics_handle& handle);
-  uint8NDArray  do_getPixels (const graphics_handle& handle);
+  ~GLWidget ();
+
+  void initializeGL ();
+
+  void draw (graphics_object go);
+  uint8NDArray  do_getPixels (graphics_object go);
   void do_print (const QString& file_cmd, const QString& term,
-                 const graphics_handle& handle);
+                 graphics_object go);
   void drawZoomBox (const QPoint& p1, const QPoint& p2);
   void resize (int /* x */, int /* y */,
                int /* width */, int /* height */) { }
   graphics_object selectFromAxes (const graphics_object& ax,
                                   const QPoint& pt);
-  QWidget * qWidget (void) { return this; }
+
+  bool begin_rendering ();
+  void end_rendering ();
 
 protected:
-  void paintGL (void);
+
+  void paintGL ();
   void mouseDoubleClickEvent (QMouseEvent *event);
   void mouseMoveEvent (QMouseEvent *event);
   void mousePressEvent (QMouseEvent *event);
@@ -88,16 +76,42 @@ protected:
 
 private:
 
-  bool begin_rendering (void);
-  void end_rendering (void);
+  Canvas& m_parent_canvas;
 
-  octave::qopengl_functions m_glfcns;
-  octave::opengl_renderer m_renderer;
+  qopengl_functions m_glfcns;
+  opengl_renderer m_renderer;
 
-#  if defined (HAVE_QT_OFFSCREEN)
   QOpenGLContext m_os_context;
   QOffscreenSurface m_os_surface;
-#  endif
+};
+
+class GLCanvas : public Canvas
+{
+public:
+
+  GLCanvas (octave::interpreter& interp, const graphics_handle& handle,
+            QWidget *parent);
+
+  ~GLCanvas ();
+
+  void draw (const graphics_handle& handle);
+  uint8NDArray  do_getPixels (const graphics_handle& handle);
+  void do_print (const QString& file_cmd, const QString& term,
+                 const graphics_handle& handle);
+  void drawZoomBox (const QPoint& p1, const QPoint& p2);
+  void resize (int /* x */, int /* y */,
+               int /* width */, int /* height */) { }
+  graphics_object selectFromAxes (const graphics_object& ax,
+                                  const QPoint& pt);
+
+  QWidget * qWidget () { return m_glwidget; }
+
+private:
+
+  GLWidget *m_glwidget;
+
+  bool begin_rendering ();
+  void end_rendering ();
 };
 
 OCTAVE_END_NAMESPACE(octave)
