@@ -31,14 +31,13 @@
 #include <stdlib.h>
 
 // Qt
-#include <QApplication>
-#include <QtCore/QByteRef>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QRegExp>
-#include <QtCore/QStringList>
 #include <QtCore>
 
+#include <QApplication>
+#include <QDir>
+#include <QFile>
+#include <QStringList>
+
 #include "unix/TerminalView.h"
 #include "unix/Vt102Emulation.h"
 
@@ -131,6 +130,11 @@ void TerminalModel::addView(TerminalView* widget)
 
         widget->setUsesMouse( _emulation->programUsesMouse() );
 
+        connect( _emulation , SIGNAL(programBracketedPasteModeChanged(bool)) ,
+                 widget , SLOT(setBracketedPasteMode(bool)) );
+
+        widget->setBracketedPasteMode(_emulation->programBracketedPasteMode());
+
         widget->setScreenWindow(_emulation->createWindow());
     }
 
