@@ -1,31 +1,35 @@
-/*
-
-Copyright (C) 1996-2016 John W. Eaton
-Copyright (C) 2009-2010 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2024 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
 #endif
 
-#include <iostream>
+#include <clocale>
+#include <istream>
+#include <ostream>
 #include <vector>
 
 #include "dNDArray.h"
@@ -75,29 +79,29 @@ default_numeric_demotion_function (const octave_base_value& a)
 }
 
 octave_base_value::type_conv_info
-octave_complex_matrix::numeric_demotion_function (void) const
+octave_complex_matrix::numeric_demotion_function () const
 {
   return octave_base_value::type_conv_info
-           (default_numeric_demotion_function,
-            octave_float_complex_matrix::static_type_id ());
+         (default_numeric_demotion_function,
+          octave_float_complex_matrix::static_type_id ());
 }
 
 octave_base_value *
-octave_complex_matrix::try_narrowing_conversion (void)
+octave_complex_matrix::try_narrowing_conversion ()
 {
-  octave_base_value *retval = 0;
+  octave_base_value *retval = nullptr;
 
-  if (matrix.numel () == 1)
+  if (m_matrix.numel () == 1)
     {
-      Complex c = matrix (0);
+      Complex c = m_matrix (0);
 
       if (c.imag () == 0.0)
         retval = new octave_scalar (c.real ());
       else
         retval = new octave_complex (c);
     }
-  else if (matrix.all_elements_are_real ())
-    retval = new octave_matrix (::real (matrix));
+  else if (m_matrix.all_elements_are_real ())
+    retval = new octave_matrix (::real (m_matrix));
 
   return retval;
 }
@@ -115,7 +119,7 @@ octave_complex_matrix::double_value (bool force_conversion) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "real scalar");
 
-  return octave::math::real (matrix(0, 0));
+  return std::real (m_matrix(0, 0));
 }
 
 float
@@ -131,7 +135,7 @@ octave_complex_matrix::float_value (bool force_conversion) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "real scalar");
 
-  return octave::math::real (matrix(0, 0));
+  return std::real (m_matrix(0, 0));
 }
 
 NDArray
@@ -143,7 +147,7 @@ octave_complex_matrix::array_value (bool force_conversion) const
     warn_implicit_conversion ("Octave:imag-to-real",
                               "complex matrix", "real matrix");
 
-  retval = ::real (matrix);
+  retval = ::real (m_matrix);
 
   return retval;
 }
@@ -157,7 +161,7 @@ octave_complex_matrix::matrix_value (bool force_conversion) const
     warn_implicit_conversion ("Octave:imag-to-real",
                               "complex matrix", "real matrix");
 
-  retval = ::real (ComplexMatrix (matrix));
+  retval = ::real (ComplexMatrix (m_matrix));
 
   return retval;
 }
@@ -171,7 +175,7 @@ octave_complex_matrix::float_matrix_value (bool force_conversion) const
     warn_implicit_conversion ("Octave:imag-to-real",
                               "complex matrix", "real matrix");
 
-  retval = ::real (ComplexMatrix (matrix));
+  retval = ::real (ComplexMatrix (m_matrix));
 
   return retval;
 }
@@ -185,7 +189,7 @@ octave_complex_matrix::complex_value (bool) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "complex scalar");
 
-  return matrix(0, 0);
+  return m_matrix(0, 0);
 }
 
 FloatComplex
@@ -201,7 +205,7 @@ octave_complex_matrix::float_complex_value (bool) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "complex scalar");
 
-  retval = matrix(0, 0);
+  retval = m_matrix(0, 0);
 
   return retval;
 }
@@ -209,25 +213,25 @@ octave_complex_matrix::float_complex_value (bool) const
 ComplexMatrix
 octave_complex_matrix::complex_matrix_value (bool) const
 {
-  return ComplexMatrix (matrix);
+  return ComplexMatrix (m_matrix);
 }
 
 FloatComplexMatrix
 octave_complex_matrix::float_complex_matrix_value (bool) const
 {
-  return FloatComplexMatrix (ComplexMatrix (matrix));
+  return FloatComplexMatrix (ComplexMatrix (m_matrix));
 }
 
 boolNDArray
 octave_complex_matrix::bool_array_value (bool warn) const
 {
-  if (matrix.any_element_is_nan ())
+  if (m_matrix.any_element_is_nan ())
     octave::err_nan_to_logical_conversion ();
-  if (warn && (! matrix.all_elements_are_real ()
-               || real (matrix).any_element_not_one_or_zero ()))
+  if (warn && (! m_matrix.all_elements_are_real ()
+               || real (m_matrix).any_element_not_one_or_zero ()))
     warn_logical_conversion ();
 
-  return mx_el_ne (matrix, Complex (0.0));
+  return mx_el_ne (m_matrix, Complex (0.0));
 }
 
 charNDArray
@@ -244,7 +248,7 @@ octave_complex_matrix::char_array_value (bool frc_str_conv) const
       octave_idx_type nel = numel ();
 
       for (octave_idx_type i = 0; i < nel; i++)
-        retval.elem (i) = static_cast<char>(octave::math::real (matrix.elem (i)));
+        retval.elem (i) = static_cast<char> (std::real (m_matrix.elem (i)));
     }
 
   return retval;
@@ -253,7 +257,7 @@ octave_complex_matrix::char_array_value (bool frc_str_conv) const
 FloatComplexNDArray
 octave_complex_matrix::float_complex_array_value (bool) const
 {
-  return FloatComplexNDArray (matrix);
+  return FloatComplexNDArray (m_matrix);
 }
 
 SparseMatrix
@@ -265,7 +269,7 @@ octave_complex_matrix::sparse_matrix_value (bool force_conversion) const
     warn_implicit_conversion ("Octave:imag-to-real",
                               "complex matrix", "real matrix");
 
-  retval = SparseMatrix (::real (ComplexMatrix (matrix)));
+  retval = SparseMatrix (::real (ComplexMatrix (m_matrix)));
 
   return retval;
 }
@@ -273,28 +277,28 @@ octave_complex_matrix::sparse_matrix_value (bool force_conversion) const
 SparseComplexMatrix
 octave_complex_matrix::sparse_complex_matrix_value (bool) const
 {
-  return SparseComplexMatrix (ComplexMatrix (matrix));
+  return SparseComplexMatrix (ComplexMatrix (m_matrix));
 }
 
 octave_value
-octave_complex_matrix::as_double (void) const
+octave_complex_matrix::as_double () const
 {
-  return matrix;
+  return m_matrix;
 }
 
 octave_value
-octave_complex_matrix::as_single (void) const
+octave_complex_matrix::as_single () const
 {
-  return FloatComplexNDArray (matrix);
+  return FloatComplexNDArray (m_matrix);
 }
 
 octave_value
 octave_complex_matrix::diag (octave_idx_type k) const
 {
   octave_value retval;
-  if (k == 0 && matrix.ndims () == 2
-      && (matrix.rows () == 1 || matrix.columns () == 1))
-    retval = ComplexDiagMatrix (DiagArray2<Complex> (matrix));
+  if (k == 0 && m_matrix.ndims () == 2
+      && (m_matrix.rows () == 1 || m_matrix.columns () == 1))
+    retval = ComplexDiagMatrix (DiagArray2<Complex> (m_matrix));
   else
     retval = octave_base_matrix<ComplexNDArray>::diag (k);
 
@@ -304,11 +308,11 @@ octave_complex_matrix::diag (octave_idx_type k) const
 octave_value
 octave_complex_matrix::diag (octave_idx_type m, octave_idx_type n) const
 {
-  if (matrix.ndims () != 2
-      || (matrix.rows () != 1 && matrix.columns () != 1))
+  if (m_matrix.ndims () != 2
+      || (m_matrix.rows () != 1 && m_matrix.columns () != 1))
     error ("diag: expecting vector argument");
 
-  ComplexMatrix mat (matrix);
+  ComplexMatrix mat (m_matrix);
 
   return mat.diag (m, n);
 }
@@ -324,7 +328,7 @@ octave_complex_matrix::save_ascii (std::ostream& os)
       os << "# ndims: " << dv.ndims () << "\n";
 
       for (int i = 0; i < dv.ndims (); i++)
-        os << " " << dv(i);
+        os << ' ' << dv(i);
 
       os << "\n" << tmp;
     }
@@ -355,6 +359,15 @@ octave_complex_matrix::load_ascii (std::istream& is)
   if (! extract_keyword (is, keywords, kw, val, true))
     error ("load: failed to extract number of rows and columns");
 
+  // Set "C" locale for the duration of this function to avoid the performance
+  // panelty of frequently switching the locale when reading floating point
+  // values from the stream.
+  char *prev_locale = std::setlocale (LC_ALL, nullptr);
+  std::string old_locale (prev_locale ? prev_locale : "");
+  std::setlocale (LC_ALL, "C");
+  octave::unwind_action act
+  ([&old_locale] () { std::setlocale (LC_ALL, old_locale.c_str ()); });
+
   if (kw == "ndims")
     {
       int mdims = static_cast<int> (val);
@@ -375,10 +388,10 @@ octave_complex_matrix::load_ascii (std::istream& is)
 
       is >> tmp;
 
-      if (!is)
+      if (! is)
         error ("load: failed to load matrix constant");
 
-      matrix = tmp;
+      m_matrix = tmp;
     }
   else if (kw == "rows")
     {
@@ -395,10 +408,10 @@ octave_complex_matrix::load_ascii (std::istream& is)
           if (! is)
             error ("load: failed to load matrix constant");
 
-          matrix = tmp;
+          m_matrix = tmp;
         }
       else if (nr == 0 || nc == 0)
-        matrix = ComplexMatrix (nr, nc);
+        m_matrix = ComplexMatrix (nr, nc);
       else
         panic_impossible ();
     }
@@ -409,7 +422,7 @@ octave_complex_matrix::load_ascii (std::istream& is)
 }
 
 bool
-octave_complex_matrix::save_binary (std::ostream& os, bool& save_as_floats)
+octave_complex_matrix::save_binary (std::ostream& os, bool save_as_floats)
 {
   dim_vector dv = dims ();
   if (dv.ndims () < 1)
@@ -440,7 +453,7 @@ octave_complex_matrix::save_binary (std::ostream& os, bool& save_as_floats)
     {
       double max_val, min_val;
       if (m.all_integers (max_val, min_val))
-        st = get_save_type (max_val, min_val);
+        st = octave::get_save_type (max_val, min_val);
     }
 
   const Complex *mtmp = m.data ();
@@ -499,7 +512,7 @@ octave_complex_matrix::load_binary (std::istream& is, bool swap,
       if (! is)
         return false;
 
-      matrix = m;
+      m_matrix = m;
     }
   else
     {
@@ -513,14 +526,14 @@ octave_complex_matrix::load_binary (std::istream& is, bool swap,
         return false;
       ComplexMatrix m (nr, nc);
       Complex *im = m.fortran_vec ();
-      octave_idx_type len = nr * nc;
+      octave_idx_type len = static_cast<octave_idx_type> (nr) * nc;
       read_doubles (is, reinterpret_cast<double *> (im),
                     static_cast<save_type> (tmp), 2*len, swap, fmt);
 
       if (! is)
         return false;
 
-      matrix = m;
+      m_matrix = m;
     }
   return true;
 }
@@ -548,7 +561,7 @@ octave_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
   for (int i = 0; i < rank; i++)
     hdims[i] = dv(rank-i-1);
 
-  space_hid = H5Screate_simple (rank, hdims, 0);
+  space_hid = H5Screate_simple (rank, hdims, nullptr);
   if (space_hid < 0) return false;
 
   hid_t save_type_hid = H5T_NATIVE_DOUBLE;
@@ -571,7 +584,7 @@ octave_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
 
       if (m.all_integers (max_val, min_val))
         save_type_hid
-          = save_type_to_hdf5 (get_save_type (max_val, min_val));
+          = save_type_to_hdf5 (octave::get_save_type (max_val, min_val));
     }
 #endif
 
@@ -583,7 +596,8 @@ octave_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
     }
 #if defined (HAVE_HDF5_18)
   data_hid = H5Dcreate (loc_id, name, type_hid, space_hid,
-                        octave_H5P_DEFAULT, octave_H5P_DEFAULT, octave_H5P_DEFAULT);
+                        octave_H5P_DEFAULT, octave_H5P_DEFAULT,
+                        octave_H5P_DEFAULT);
 #else
   data_hid = H5Dcreate (loc_id, name, type_hid, space_hid, octave_H5P_DEFAULT);
 #endif
@@ -599,7 +613,7 @@ octave_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
 
   if (retval)
     {
-      Complex *mtmp = m.fortran_vec ();
+      const Complex *mtmp = m.data ();
       if (H5Dwrite (data_hid, complex_type_hid, octave_H5S_ALL, octave_H5S_ALL,
                     octave_H5P_DEFAULT, mtmp)
           < 0)
@@ -637,7 +651,7 @@ octave_complex_matrix::load_hdf5 (octave_hdf5_id loc_id, const char *name)
   dim_vector dv;
   int empty = load_hdf5_empty (loc_id, name, dv);
   if (empty > 0)
-    matrix.resize (dv);
+    m_matrix.resize (dv);
   if (empty)
     return (empty > 0);
 
@@ -695,7 +709,7 @@ octave_complex_matrix::load_hdf5 (octave_hdf5_id loc_id, const char *name)
       >= 0)
     {
       retval = true;
-      matrix = m;
+      m_matrix = m;
     }
 
   H5Tclose (complex_type);
@@ -716,26 +730,41 @@ void
 octave_complex_matrix::print_raw (std::ostream& os,
                                   bool pr_as_read_syntax) const
 {
-  octave_print_internal (os, matrix, pr_as_read_syntax,
+  octave_print_internal (os, m_matrix, pr_as_read_syntax,
                          current_print_indent_level ());
 }
 
 mxArray *
-octave_complex_matrix::as_mxArray (void) const
+octave_complex_matrix::as_mxArray (bool interleaved) const
 {
-  mxArray *retval = new mxArray (mxDOUBLE_CLASS, dims (), mxCOMPLEX);
-
-  double *pr = static_cast<double *> (retval->get_data ());
-  double *pi = static_cast<double *> (retval->get_imag_data ());
+  mxArray *retval = new mxArray (interleaved, mxDOUBLE_CLASS, dims (),
+                                 mxCOMPLEX);
 
   mwSize nel = numel ();
 
-  const Complex *p = matrix.data ();
+  const Complex *pdata = m_matrix.data ();
 
-  for (mwIndex i = 0; i < nel; i++)
+  if (interleaved)
     {
-      pr[i] = octave::math::real (p[i]);
-      pi[i] = octave::math::imag (p[i]);
+      mxComplexDouble *pd
+        = static_cast<mxComplexDouble *> (retval->get_data ());
+
+      for (mwIndex i = 0; i < nel; i++)
+        {
+          pd[i].real = pdata[i].real ();
+          pd[i].imag = pdata[i].imag ();
+        }
+    }
+  else
+    {
+      mxDouble *pr = static_cast<mxDouble *> (retval->get_data ());
+      mxDouble *pi = static_cast<mxDouble *> (retval->get_imag_data ());
+
+      for (mwIndex i = 0; i < nel; i++)
+        {
+          pr[i] = pdata[i].real ();
+          pi[i] = pdata[i].imag ();
+        }
     }
 
   return retval;
@@ -748,61 +777,65 @@ octave_complex_matrix::map (unary_mapper_t umap) const
     {
     // Mappers handled specially.
     case umap_real:
-      return ::real (matrix);
+      return ::real (m_matrix);
     case umap_imag:
-      return ::imag (matrix);
+      return ::imag (m_matrix);
     case umap_conj:
-      return ::conj (matrix);
+      return ::conj (m_matrix);
+
+    // Special cases for Matlab compatibility.
+    case umap_xtolower:
+    case umap_xtoupper:
+      return m_matrix;
 
 #define ARRAY_METHOD_MAPPER(UMAP, FCN)        \
     case umap_ ## UMAP:                       \
-      return octave_value (matrix.FCN ())
+      return octave_value (m_matrix.FCN ())
 
-    ARRAY_METHOD_MAPPER (abs, abs);
-    ARRAY_METHOD_MAPPER (isnan, isnan);
-    ARRAY_METHOD_MAPPER (isinf, isinf);
-    ARRAY_METHOD_MAPPER (isfinite, isfinite);
+      ARRAY_METHOD_MAPPER (abs, abs);
+      ARRAY_METHOD_MAPPER (isnan, isnan);
+      ARRAY_METHOD_MAPPER (isinf, isinf);
+      ARRAY_METHOD_MAPPER (isfinite, isfinite);
 
 #define ARRAY_MAPPER(UMAP, TYPE, FCN)                 \
     case umap_ ## UMAP:                               \
-      return octave_value (matrix.map<TYPE> (FCN))
+      return octave_value (m_matrix.map<TYPE> (FCN))
 
-    ARRAY_MAPPER (acos, Complex, octave::math::acos);
-    ARRAY_MAPPER (acosh, Complex, octave::math::acosh);
-    ARRAY_MAPPER (angle, double, std::arg);
-    ARRAY_MAPPER (arg, double, std::arg);
-    ARRAY_MAPPER (asin, Complex, octave::math::asin);
-    ARRAY_MAPPER (asinh, Complex, octave::math::asinh);
-    ARRAY_MAPPER (atan, Complex, octave::math::atan);
-    ARRAY_MAPPER (atanh, Complex, octave::math::atanh);
-    ARRAY_MAPPER (erf, Complex, octave::math::erf);
-    ARRAY_MAPPER (erfc, Complex, octave::math::erfc);
-    ARRAY_MAPPER (erfcx, Complex, octave::math::erfcx);
-    ARRAY_MAPPER (erfi, Complex, octave::math::erfi);
-    ARRAY_MAPPER (dawson, Complex, octave::math::dawson);
-    ARRAY_MAPPER (ceil, Complex, octave::math::ceil);
-    ARRAY_MAPPER (cos, Complex, std::cos);
-    ARRAY_MAPPER (cosh, Complex, std::cosh);
-    ARRAY_MAPPER (exp, Complex, std::exp);
-    ARRAY_MAPPER (expm1, Complex, octave::math::expm1);
-    ARRAY_MAPPER (fix, Complex, octave::math::fix);
-    ARRAY_MAPPER (floor, Complex, octave::math::floor);
-    ARRAY_MAPPER (log, Complex, std::log);
-    ARRAY_MAPPER (log2, Complex, octave::math::log2);
-    ARRAY_MAPPER (log10, Complex, std::log10);
-    ARRAY_MAPPER (log1p, Complex, octave::math::log1p);
-    ARRAY_MAPPER (round, Complex, octave::math::round);
-    ARRAY_MAPPER (roundb, Complex, octave::math::roundb);
-    ARRAY_MAPPER (signum, Complex, octave::math::signum);
-    ARRAY_MAPPER (sin, Complex, std::sin);
-    ARRAY_MAPPER (sinh, Complex, std::sinh);
-    ARRAY_MAPPER (sqrt, Complex, std::sqrt);
-    ARRAY_MAPPER (tan, Complex, std::tan);
-    ARRAY_MAPPER (tanh, Complex, std::tanh);
-    ARRAY_MAPPER (isna, bool, octave::math::is_NA);
+      ARRAY_MAPPER (acos, Complex, octave::math::acos);
+      ARRAY_MAPPER (acosh, Complex, octave::math::acosh);
+      ARRAY_MAPPER (angle, double, std::arg);
+      ARRAY_MAPPER (arg, double, std::arg);
+      ARRAY_MAPPER (asin, Complex, octave::math::asin);
+      ARRAY_MAPPER (asinh, Complex, octave::math::asinh);
+      ARRAY_MAPPER (atan, Complex, octave::math::atan);
+      ARRAY_MAPPER (atanh, Complex, octave::math::atanh);
+      ARRAY_MAPPER (erf, Complex, octave::math::erf);
+      ARRAY_MAPPER (erfc, Complex, octave::math::erfc);
+      ARRAY_MAPPER (erfcx, Complex, octave::math::erfcx);
+      ARRAY_MAPPER (erfi, Complex, octave::math::erfi);
+      ARRAY_MAPPER (dawson, Complex, octave::math::dawson);
+      ARRAY_MAPPER (ceil, Complex, octave::math::ceil);
+      ARRAY_MAPPER (cos, Complex, std::cos);
+      ARRAY_MAPPER (cosh, Complex, std::cosh);
+      ARRAY_MAPPER (exp, Complex, std::exp);
+      ARRAY_MAPPER (expm1, Complex, octave::math::expm1);
+      ARRAY_MAPPER (fix, Complex, octave::math::fix);
+      ARRAY_MAPPER (floor, Complex, octave::math::floor);
+      ARRAY_MAPPER (log, Complex, std::log);
+      ARRAY_MAPPER (log2, Complex, octave::math::log2);
+      ARRAY_MAPPER (log10, Complex, std::log10);
+      ARRAY_MAPPER (log1p, Complex, octave::math::log1p);
+      ARRAY_MAPPER (round, Complex, octave::math::round);
+      ARRAY_MAPPER (roundb, Complex, octave::math::roundb);
+      ARRAY_MAPPER (signum, Complex, octave::math::signum);
+      ARRAY_MAPPER (sin, Complex, std::sin);
+      ARRAY_MAPPER (sinh, Complex, std::sinh);
+      ARRAY_MAPPER (sqrt, Complex, std::sqrt);
+      ARRAY_MAPPER (tan, Complex, std::tan);
+      ARRAY_MAPPER (tanh, Complex, std::tanh);
+      ARRAY_MAPPER (isna, bool, octave::math::isna);
 
     default:
       return octave_base_value::map (umap);
     }
 }
-
