@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2020 The Octave Project Developers
+// Copyright (C) 1996-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -27,6 +27,7 @@
 #  include "config.h"
 #endif
 
+#include <clocale>
 #include <istream>
 #include <ostream>
 #include <vector>
@@ -74,21 +75,21 @@ DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_float_complex_matrix,
                                      "float complex matrix", "single");
 
 octave_base_value *
-octave_float_complex_matrix::try_narrowing_conversion (void)
+octave_float_complex_matrix::try_narrowing_conversion ()
 {
   octave_base_value *retval = nullptr;
 
-  if (matrix.numel () == 1)
+  if (m_matrix.numel () == 1)
     {
-      FloatComplex c = matrix (0);
+      FloatComplex c = m_matrix (0);
 
       if (c.imag () == 0.0)
         retval = new octave_float_scalar (c.real ());
       else
         retval = new octave_float_complex (c);
     }
-  else if (matrix.all_elements_are_real ())
-    retval = new octave_float_matrix (::real (matrix));
+  else if (m_matrix.all_elements_are_real ())
+    retval = new octave_float_matrix (::real (m_matrix));
 
   return retval;
 }
@@ -106,7 +107,7 @@ octave_float_complex_matrix::double_value (bool force_conversion) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "real scalar");
 
-  return std::real (matrix(0, 0));
+  return std::real (m_matrix(0, 0));
 }
 
 float
@@ -122,7 +123,7 @@ octave_float_complex_matrix::float_value (bool force_conversion) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "real scalar");
 
-  return std::real (matrix(0, 0));
+  return std::real (m_matrix(0, 0));
 }
 
 Matrix
@@ -134,7 +135,7 @@ octave_float_complex_matrix::matrix_value (bool force_conversion) const
     warn_implicit_conversion ("Octave:imag-to-real",
                               "complex matrix", "real matrix");
 
-  retval = ::real (FloatComplexMatrix (matrix));
+  retval = ::real (FloatComplexMatrix (m_matrix));
 
   return retval;
 }
@@ -148,7 +149,7 @@ octave_float_complex_matrix::float_matrix_value (bool force_conversion) const
     warn_implicit_conversion ("Octave:imag-to-real",
                               "complex matrix", "real matrix");
 
-  retval = ::real (FloatComplexMatrix (matrix));
+  retval = ::real (FloatComplexMatrix (m_matrix));
 
   return retval;
 }
@@ -162,7 +163,7 @@ octave_float_complex_matrix::complex_value (bool) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "complex scalar");
 
-  return matrix(0, 0);
+  return m_matrix(0, 0);
 }
 
 FloatComplex
@@ -178,7 +179,7 @@ octave_float_complex_matrix::float_complex_value (bool) const
   warn_implicit_conversion ("Octave:array-to-scalar",
                             "complex matrix", "complex scalar");
 
-  retval = matrix(0, 0);
+  retval = m_matrix(0, 0);
 
   return retval;
 }
@@ -186,25 +187,25 @@ octave_float_complex_matrix::float_complex_value (bool) const
 ComplexMatrix
 octave_float_complex_matrix::complex_matrix_value (bool) const
 {
-  return FloatComplexMatrix (matrix);
+  return FloatComplexMatrix (m_matrix);
 }
 
 FloatComplexMatrix
 octave_float_complex_matrix::float_complex_matrix_value (bool) const
 {
-  return FloatComplexMatrix (matrix);
+  return FloatComplexMatrix (m_matrix);
 }
 
 boolNDArray
 octave_float_complex_matrix::bool_array_value (bool warn) const
 {
-  if (matrix.any_element_is_nan ())
+  if (m_matrix.any_element_is_nan ())
     octave::err_nan_to_logical_conversion ();
-  if (warn && (! matrix.all_elements_are_real ()
-               || real (matrix).any_element_not_one_or_zero ()))
+  if (warn && (! m_matrix.all_elements_are_real ()
+               || real (m_matrix).any_element_not_one_or_zero ()))
     warn_logical_conversion ();
 
-  return mx_el_ne (matrix, FloatComplex (0.0));
+  return mx_el_ne (m_matrix, FloatComplex (0.0));
 }
 
 charNDArray
@@ -221,7 +222,7 @@ octave_float_complex_matrix::char_array_value (bool frc_str_conv) const
       octave_idx_type nel = numel ();
 
       for (octave_idx_type i = 0; i < nel; i++)
-        retval.elem (i) = static_cast<char> (std::real (matrix.elem (i)));
+        retval.elem (i) = static_cast<char> (std::real (m_matrix.elem (i)));
     }
 
   return retval;
@@ -230,7 +231,7 @@ octave_float_complex_matrix::char_array_value (bool frc_str_conv) const
 FloatComplexNDArray
 octave_float_complex_matrix::float_complex_array_value (bool) const
 {
-  return FloatComplexNDArray (matrix);
+  return FloatComplexNDArray (m_matrix);
 }
 
 SparseMatrix
@@ -254,24 +255,24 @@ octave_float_complex_matrix::sparse_complex_matrix_value (bool) const
 }
 
 octave_value
-octave_float_complex_matrix::as_double (void) const
+octave_float_complex_matrix::as_double () const
 {
-  return ComplexNDArray (matrix);
+  return ComplexNDArray (m_matrix);
 }
 
 octave_value
-octave_float_complex_matrix::as_single (void) const
+octave_float_complex_matrix::as_single () const
 {
-  return matrix;
+  return m_matrix;
 }
 
 octave_value
 octave_float_complex_matrix::diag (octave_idx_type k) const
 {
   octave_value retval;
-  if (k == 0 && matrix.ndims () == 2
-      && (matrix.rows () == 1 || matrix.columns () == 1))
-    retval = FloatComplexDiagMatrix (DiagArray2<FloatComplex> (matrix));
+  if (k == 0 && m_matrix.ndims () == 2
+      && (m_matrix.rows () == 1 || m_matrix.columns () == 1))
+    retval = FloatComplexDiagMatrix (DiagArray2<FloatComplex> (m_matrix));
   else
     retval = octave_base_matrix<FloatComplexNDArray>::diag (k);
 
@@ -281,11 +282,11 @@ octave_float_complex_matrix::diag (octave_idx_type k) const
 octave_value
 octave_float_complex_matrix::diag (octave_idx_type m, octave_idx_type n) const
 {
-  if (matrix.ndims () != 2
-      || (matrix.rows () != 1 && matrix.columns () != 1))
+  if (m_matrix.ndims () != 2
+      || (m_matrix.rows () != 1 && m_matrix.columns () != 1))
     error ("diag: expecting vector argument");
 
-  FloatComplexMatrix mat (matrix);
+  FloatComplexMatrix mat (m_matrix);
 
   return mat.diag (m, n);
 }
@@ -294,6 +295,7 @@ bool
 octave_float_complex_matrix::save_ascii (std::ostream& os)
 {
   dim_vector dv = dims ();
+
   if (dv.ndims () > 2)
     {
       FloatComplexNDArray tmp = complex_array_value ();
@@ -332,6 +334,15 @@ octave_float_complex_matrix::load_ascii (std::istream& is)
   if (! extract_keyword (is, keywords, kw, val, true))
     error ("load: failed to extract number of rows and columns");
 
+  // Set "C" locale for the duration of this function to avoid the performance
+  // panelty of frequently switching the locale when reading floating point
+  // values from the stream.
+  char *prev_locale = std::setlocale (LC_ALL, nullptr);
+  std::string old_locale (prev_locale ? prev_locale : "");
+  std::setlocale (LC_ALL, "C");
+  octave::unwind_action act
+  ([&old_locale] () { std::setlocale (LC_ALL, old_locale.c_str ()); });
+
   if (kw == "ndims")
     {
       int mdims = static_cast<int> (val);
@@ -355,7 +366,7 @@ octave_float_complex_matrix::load_ascii (std::istream& is)
       if (! is)
         error ("load: failed to load matrix constant");
 
-      matrix = tmp;
+      m_matrix = tmp;
     }
   else if (kw == "rows")
     {
@@ -372,10 +383,10 @@ octave_float_complex_matrix::load_ascii (std::istream& is)
           if (! is)
             error ("load: failed to load matrix constant");
 
-          matrix = tmp;
+          m_matrix = tmp;
         }
       else if (nr == 0 || nc == 0)
-        matrix = FloatComplexMatrix (nr, nc);
+        m_matrix = FloatComplexMatrix (nr, nc);
       else
         panic_impossible ();
     }
@@ -407,7 +418,7 @@ octave_float_complex_matrix::save_binary (std::ostream& os, bool)
     {
       float max_val, min_val;
       if (m.all_integers (max_val, min_val))
-        st = get_save_type (max_val, min_val);
+        st = octave::get_save_type (max_val, min_val);
     }
 
   const FloatComplex *mtmp = m.data ();
@@ -419,7 +430,7 @@ octave_float_complex_matrix::save_binary (std::ostream& os, bool)
 
 bool
 octave_float_complex_matrix::load_binary (std::istream& is, bool swap,
-                                          octave::mach_info::float_format fmt)
+    octave::mach_info::float_format fmt)
 {
   char tmp;
   int32_t mdims;
@@ -466,7 +477,7 @@ octave_float_complex_matrix::load_binary (std::istream& is, bool swap,
       if (! is)
         return false;
 
-      matrix = m;
+      m_matrix = m;
     }
   else
     {
@@ -480,14 +491,14 @@ octave_float_complex_matrix::load_binary (std::istream& is, bool swap,
         return false;
       FloatComplexMatrix m (nr, nc);
       FloatComplex *im = m.fortran_vec ();
-      octave_idx_type len = nr * nc;
+      octave_idx_type len = static_cast<octave_idx_type> (nr) * nc;
       read_floats (is, reinterpret_cast<float *> (im),
                    static_cast<save_type> (tmp), 2*len, swap, fmt);
 
       if (! is)
         return false;
 
-      matrix = m;
+      m_matrix = m;
     }
   return true;
 }
@@ -529,7 +540,7 @@ octave_float_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
 
       if (m.all_integers (max_val, min_val))
         save_type_hid
-          = save_type_to_hdf5 (get_save_type (max_val, min_val));
+          = save_type_to_hdf5 (octave::get_save_type (max_val, min_val));
     }
 #endif
 
@@ -541,7 +552,8 @@ octave_float_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
     }
 #if defined (HAVE_HDF5_18)
   data_hid = H5Dcreate (loc_id, name, type_hid, space_hid,
-                        octave_H5P_DEFAULT, octave_H5P_DEFAULT, octave_H5P_DEFAULT);
+                        octave_H5P_DEFAULT, octave_H5P_DEFAULT,
+                        octave_H5P_DEFAULT);
 #else
   data_hid = H5Dcreate (loc_id, name, type_hid, space_hid, octave_H5P_DEFAULT);
 #endif
@@ -557,7 +569,7 @@ octave_float_complex_matrix::save_hdf5 (octave_hdf5_id loc_id, const char *name,
 
   if (retval)
     {
-      FloatComplex *mtmp = m.fortran_vec ();
+      const FloatComplex *mtmp = m.data ();
       if (H5Dwrite (data_hid, complex_type_hid, octave_H5S_ALL, octave_H5S_ALL,
                     octave_H5P_DEFAULT, mtmp)
           < 0)
@@ -592,7 +604,7 @@ octave_float_complex_matrix::load_hdf5 (octave_hdf5_id loc_id, const char *name)
   dim_vector dv;
   int empty = load_hdf5_empty (loc_id, name, dv);
   if (empty > 0)
-    matrix.resize (dv);
+    m_matrix.resize (dv);
   if (empty)
     return (empty > 0);
 
@@ -650,7 +662,7 @@ octave_float_complex_matrix::load_hdf5 (octave_hdf5_id loc_id, const char *name)
       >= 0)
     {
       retval = true;
-      matrix = m;
+      m_matrix = m;
     }
 
   H5Tclose (complex_type);
@@ -671,26 +683,41 @@ void
 octave_float_complex_matrix::print_raw (std::ostream& os,
                                         bool pr_as_read_syntax) const
 {
-  octave_print_internal (os, matrix, pr_as_read_syntax,
+  octave_print_internal (os, m_matrix, pr_as_read_syntax,
                          current_print_indent_level ());
 }
 
 mxArray *
-octave_float_complex_matrix::as_mxArray (void) const
+octave_float_complex_matrix::as_mxArray (bool interleaved) const
 {
-  mxArray *retval = new mxArray (mxSINGLE_CLASS, dims (), mxCOMPLEX);
-
-  float *pr = static_cast<float *> (retval->get_data ());
-  float *pi = static_cast<float *> (retval->get_imag_data ());
+  mxArray *retval = new mxArray (interleaved, mxSINGLE_CLASS, dims (),
+                                 mxCOMPLEX);
 
   mwSize nel = numel ();
 
-  const FloatComplex *p = matrix.data ();
+  const FloatComplex *pdata = m_matrix.data ();
 
-  for (mwIndex i = 0; i < nel; i++)
+  if (interleaved)
     {
-      pr[i] = std::real (p[i]);
-      pi[i] = std::imag (p[i]);
+      mxComplexSingle *pd
+        = static_cast<mxComplexSingle *> (retval->get_data ());
+
+      for (mwIndex i = 0; i < nel; i++)
+        {
+          pd[i].real = pdata[i].real ();
+          pd[i].imag = pdata[i].imag ();
+        }
+    }
+  else
+    {
+      mxSingle *pr = static_cast<mxSingle *> (retval->get_data ());
+      mxSingle *pi = static_cast<mxSingle *> (retval->get_imag_data ());
+
+      for (mwIndex i = 0; i < nel; i++)
+        {
+          pr[i] = pdata[i].real ();
+          pi[i] = pdata[i].imag ();
+        }
     }
 
   return retval;
@@ -703,63 +730,63 @@ octave_float_complex_matrix::map (unary_mapper_t umap) const
     {
     // Mappers handled specially.
     case umap_real:
-      return ::real (matrix);
+      return ::real (m_matrix);
     case umap_imag:
-      return ::imag (matrix);
+      return ::imag (m_matrix);
     case umap_conj:
-      return ::conj (matrix);
+      return ::conj (m_matrix);
 
     // Special cases for Matlab compatibility.
     case umap_xtolower:
     case umap_xtoupper:
-      return matrix;
+      return m_matrix;
 
 #define ARRAY_METHOD_MAPPER(UMAP, FCN)        \
     case umap_ ## UMAP:                       \
-      return octave_value (matrix.FCN ())
+      return octave_value (m_matrix.FCN ())
 
-    ARRAY_METHOD_MAPPER (abs, abs);
-    ARRAY_METHOD_MAPPER (isnan, isnan);
-    ARRAY_METHOD_MAPPER (isinf, isinf);
-    ARRAY_METHOD_MAPPER (isfinite, isfinite);
+      ARRAY_METHOD_MAPPER (abs, abs);
+      ARRAY_METHOD_MAPPER (isnan, isnan);
+      ARRAY_METHOD_MAPPER (isinf, isinf);
+      ARRAY_METHOD_MAPPER (isfinite, isfinite);
 
 #define ARRAY_MAPPER(UMAP, TYPE, FCN)                 \
     case umap_ ## UMAP:                               \
-      return octave_value (matrix.map<TYPE> (FCN))
+      return octave_value (m_matrix.map<TYPE> (FCN))
 
-    ARRAY_MAPPER (acos, FloatComplex, octave::math::acos);
-    ARRAY_MAPPER (acosh, FloatComplex, octave::math::acosh);
-    ARRAY_MAPPER (angle, float, std::arg);
-    ARRAY_MAPPER (arg, float, std::arg);
-    ARRAY_MAPPER (asin, FloatComplex, octave::math::asin);
-    ARRAY_MAPPER (asinh, FloatComplex, octave::math::asinh);
-    ARRAY_MAPPER (atan, FloatComplex, octave::math::atan);
-    ARRAY_MAPPER (atanh, FloatComplex, octave::math::atanh);
-    ARRAY_MAPPER (erf, FloatComplex, octave::math::erf);
-    ARRAY_MAPPER (erfc, FloatComplex, octave::math::erfc);
-    ARRAY_MAPPER (erfcx, FloatComplex, octave::math::erfcx);
-    ARRAY_MAPPER (erfi, FloatComplex, octave::math::erfi);
-    ARRAY_MAPPER (dawson, FloatComplex, octave::math::dawson);
-    ARRAY_MAPPER (ceil, FloatComplex, octave::math::ceil);
-    ARRAY_MAPPER (cos, FloatComplex, std::cos);
-    ARRAY_MAPPER (cosh, FloatComplex, std::cosh);
-    ARRAY_MAPPER (exp, FloatComplex, std::exp);
-    ARRAY_MAPPER (expm1, FloatComplex, octave::math::expm1);
-    ARRAY_MAPPER (fix, FloatComplex, octave::math::fix);
-    ARRAY_MAPPER (floor, FloatComplex, octave::math::floor);
-    ARRAY_MAPPER (log, FloatComplex, std::log);
-    ARRAY_MAPPER (log2, FloatComplex, octave::math::log2);
-    ARRAY_MAPPER (log10, FloatComplex, std::log10);
-    ARRAY_MAPPER (log1p, FloatComplex, octave::math::log1p);
-    ARRAY_MAPPER (round, FloatComplex, octave::math::round);
-    ARRAY_MAPPER (roundb, FloatComplex, octave::math::roundb);
-    ARRAY_MAPPER (signum, FloatComplex, octave::math::signum);
-    ARRAY_MAPPER (sin, FloatComplex, std::sin);
-    ARRAY_MAPPER (sinh, FloatComplex, std::sinh);
-    ARRAY_MAPPER (sqrt, FloatComplex, std::sqrt);
-    ARRAY_MAPPER (tan, FloatComplex, std::tan);
-    ARRAY_MAPPER (tanh, FloatComplex, std::tanh);
-    ARRAY_MAPPER (isna, bool, octave::math::isna);
+      ARRAY_MAPPER (acos, FloatComplex, octave::math::acos);
+      ARRAY_MAPPER (acosh, FloatComplex, octave::math::acosh);
+      ARRAY_MAPPER (angle, float, std::arg);
+      ARRAY_MAPPER (arg, float, std::arg);
+      ARRAY_MAPPER (asin, FloatComplex, octave::math::asin);
+      ARRAY_MAPPER (asinh, FloatComplex, octave::math::asinh);
+      ARRAY_MAPPER (atan, FloatComplex, octave::math::atan);
+      ARRAY_MAPPER (atanh, FloatComplex, octave::math::atanh);
+      ARRAY_MAPPER (erf, FloatComplex, octave::math::erf);
+      ARRAY_MAPPER (erfc, FloatComplex, octave::math::erfc);
+      ARRAY_MAPPER (erfcx, FloatComplex, octave::math::erfcx);
+      ARRAY_MAPPER (erfi, FloatComplex, octave::math::erfi);
+      ARRAY_MAPPER (dawson, FloatComplex, octave::math::dawson);
+      ARRAY_MAPPER (ceil, FloatComplex, octave::math::ceil);
+      ARRAY_MAPPER (cos, FloatComplex, std::cos);
+      ARRAY_MAPPER (cosh, FloatComplex, std::cosh);
+      ARRAY_MAPPER (exp, FloatComplex, std::exp);
+      ARRAY_MAPPER (expm1, FloatComplex, octave::math::expm1);
+      ARRAY_MAPPER (fix, FloatComplex, octave::math::fix);
+      ARRAY_MAPPER (floor, FloatComplex, octave::math::floor);
+      ARRAY_MAPPER (log, FloatComplex, std::log);
+      ARRAY_MAPPER (log2, FloatComplex, octave::math::log2);
+      ARRAY_MAPPER (log10, FloatComplex, std::log10);
+      ARRAY_MAPPER (log1p, FloatComplex, octave::math::log1p);
+      ARRAY_MAPPER (round, FloatComplex, octave::math::round);
+      ARRAY_MAPPER (roundb, FloatComplex, octave::math::roundb);
+      ARRAY_MAPPER (signum, FloatComplex, octave::math::signum);
+      ARRAY_MAPPER (sin, FloatComplex, std::sin);
+      ARRAY_MAPPER (sinh, FloatComplex, std::sinh);
+      ARRAY_MAPPER (sqrt, FloatComplex, std::sqrt);
+      ARRAY_MAPPER (tan, FloatComplex, std::tan);
+      ARRAY_MAPPER (tanh, FloatComplex, std::tanh);
+      ARRAY_MAPPER (isna, bool, octave::math::isna);
 
     default:
       return octave_base_value::map (umap);
