@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2016-2021 The Octave Project Developers
+// Copyright (C) 2016-2024 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -30,11 +30,9 @@
 extern "C" {
 #endif
 
-double
-octave_frexp_wrapper (double x, int *expptr);
+extern OCTAVE_API double octave_frexp_wrapper (double x, int *expptr);
 
-float
-octave_frexpf_wrapper (float x, int *expptr);
+extern OCTAVE_API float octave_frexpf_wrapper (float x, int *expptr);
 
 #if defined __cplusplus
 }
