@@ -1,20 +1,27 @@
-## Copyright (C) 1998-2017 Ariel Tankus
+########################################################################
+##
+## Copyright (C) 1998-2024 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {[@var{r1}, @var{r2}, @dots{}, @var{rn}] =} deal (@var{a})
@@ -56,7 +63,7 @@
 ## a = b = c = x;
 ## @end example
 ##
-## Programming Note: @code{deal} is often used with comma separated lists
+## Programming Note: @code{deal} is often used with comma-separated lists
 ## derived from cell arrays or structures.  This is unnecessary as the
 ## interpreter can perform the same action without the overhead of a function
 ## call.  For example:
@@ -77,12 +84,7 @@
 ## @seealso{cell2struct, struct2cell, repmat}
 ## @end deftypefn
 
-## Author: Ariel Tankus
-## Author: Paul Kienzle and Etienne Grossman
-## Created: 13.11.98
-## Adapted-by: jwe
-
-function [varargout] = deal (varargin)
+function varargout = deal (varargin)
 
   if (nargin == 0)
     print_usage ();
@@ -104,5 +106,6 @@ endfunction
 %! assert (a, 1);
 %! assert (b, 1);
 
-%!error deal ()
+## Test input validation
+%!error <Invalid call> deal ()
 %!error <nargin . 1 and nargin != nargout> y = deal (1, 2)
